#!/bin/sh
#\
exec wish "$0" "$@"

# Gimp plug-in for Genius ColorPage-CS Desktop Scanner
# (C) Copyright 1997, by Rildo Pragana <rpragana@acm.org>
# S-mail: Caixa Postal 7440, Recife Brazil 50,630-970
#
# This is free software, following Free Software Foundation's GNU
# philosophy. Please read GNU Copyright statements.
# You may freely modify, use in commercial or personal software, but
# please, acknowledge its author. You may also ask me for changes and 
# improvements. Feel free to contact me.
#

# Here are some options you may like to change to fit your needs

set tmpfile "/tmp/tclscan_preview.pnm"	;# this is where to put previewing 
set savefile "/tmp/scanimg.pnm"   ;# this is where the final image goes


# Try to load Gimptcllib

set Gimptcllib Gimptcl.so
set verbose 0

set path {
    $env(HOME)/.gimp/plug-ins
    /usr/X11R6/libexec/gimp*
    /usr/X11R6/lib/gimp*
    /usr/local/libexec/gimp*
    /usr/local/lib/gimp*
    $env(HOME)
}

set gtcl_loaded "notyet"

foreach dir $path {
    if {$verbose} {puts -nonewline "looking in $dir"}
    if {![catch "glob $dir/$Gimptcllib" fqp]} {
    if {$verbose} {puts " found."}
	set gtcl_loaded "ok"
	load [lindex $fqp 0]
	break
    }
    if {$verbose} {puts "."}
}

if {[string compare $gtcl_loaded "notyet"] == 0} {
    return -code error "couldn't find $Gimptcllib"
}

# I've sent the scanned image to a predefined page, because I use
# invariably "The GIMP" to postprocess the image before saving it
# to a new file.  You may change this behavior if you like.

set box(last) ""
set resolution 25
set sc(left) 0
set sc(top) 0
set sc(width) 85
set sc(height) 117

proc rescan { } {
	global im tmpfile
	exec gscan -o$tmpfile
	set im [image create photo -file $tmpfile]
	.c itemconfigure preview -image $im
}

proc doscan { } {
	global im resolution sc savefile
	exec gscan -o$savefile -r$resolution -l$sc(left) -t$sc(top) \
		-w$sc(width) -h$sc(height)
	gimp-display-new [gimp-file-load $savefile $savefile]
}

proc Mark { x y w } {
	global box
	
	catch [$w delete $box(last)] ;# remove previous box
	set box(anchor) [list $x $y]
#	catch {unset box(last)}
	$w config -cursor ul_angle
}

proc Drag { x y w } {
	global box

	catch {$w delete $box(last)}
	set box(last) [eval {$w create rect} $box(anchor) \
		{$x $y -tag box}]
	$w config -cursor lr_angle
}

proc DragRelease { x y w } {
	global box resolution sc
	
	$w config -cursor hand2

	set x0 [lindex $box(anchor) 0]
	set y0 [lindex $box(anchor) 1]
	set sc(left) [expr (($x0 - 5) * 10) / 25]
	set sc(top) [expr (($y0 - 5) * 10) / 25]
	set sc(width) [expr (($x - $x0) * 10) / 25]
	set sc(height) [expr (($y - $y0) * 10) / 25]
	
	puts "left = $sc(left), top = $sc(top), \
		 width = $sc(width), \
		 height = $sc(height)"
}

proc updlabel { } {
	global resolution
	.f.mb configure -text "$resolution dpi"	
}


proc scan-toplevel { } {
	global tmpfile resolution
	
	if [file exists $tmpfile] {
		set im [image create photo -file $tmpfile]
	} else {
		set im [image create photo -width 212 -height 292]
	}
	canvas .c -width 212 -height 292 -bd 5 -relief ridge
	.c create image 5 5 -image $im -tag preview -anchor nw
	.c bind preview <Button-1> {Mark %x %y %W}
	.c bind preview <B1-Motion> {Drag %x %y %W}
	.c bind preview <ButtonRelease-1> {DragRelease %x %y %W}
	.c config -cursor hand2

	frame .f -relief sunken -bd 3 -bg bisque3
	menubutton .f.mb -text "resolution" -menu .f.mb.m -relief raised -indicatoron true
	button .f.b -text preview -command rescan 
	button .f.s -text scan -command doscan -activebackground HotPink
	button .f.exit -command "destroy ." -text Quit
	menu .f.mb.m
	.f.mb.m add radiobutton -label "25 dpi" -variable resolution \
		-value 25 -command updlabel
	.f.mb.m add radiobutton -label "50 dpi" -variable resolution \
		-value 50 -command updlabel
	.f.mb.m add radiobutton -label "75 dpi" -variable resolution \
		-value 75 -command updlabel
	.f.mb.m add radiobutton -label "100 dpi" -variable resolution \
		-value 100 -command updlabel
	.f.mb.m add radiobutton -label "150 dpi" -variable resolution \
		-value 150 -command updlabel
	.f.mb.m add radiobutton -label "200 dpi" -variable resolution \
		-value 200 -command updlabel
	.f.mb.m add radiobutton -label "250 dpi" -variable resolution \
		-value 250 -command updlabel
	.f.mb.m add radiobutton -label "300 dpi" -variable resolution \
		-value 300 -command updlabel


	pack .f.b .f.mb .f.s .f.exit -padx 5 -pady 5 
	pack .c .f -side left
}

# Gimp interface procedures

proc gimptcl_query {} {
    gimp-install-procedure "genius_scan" \
	"Create scanned images by Genius ColorPage-CS" \
	"None Yet" \
	"Rildo Pragana" \
	"Rildo Pragana" \
	"1997" \
        "<Toolbox>/Extensions/Genius Scan" \
	"RGB" \
	extension \
	{
	    {int32 "run_mode" "Interactive, Non-interactive"}
	} \
	{}
}

proc gimptcl_run {mode} {
    switch $mode {
	0 {
	    scan-toplevel
	}
	1 {	}
	2 {	}
    }
}

gimp-main

# this is the END
